﻿// <copyright file="ConsoleClient.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
namespace CloudDigitalAssetAccessConsoleClient
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.IO;
    using System.Collections.Specialized;
    using CloudDigitalAssetAccessConsoleClient.CloudDigitalStorageService;
  
    /// <summary>
    /// This Sample console client is using Web services to communicate with Azure services (CloudDigitalAssetStorageWebService)
    /// </summary>
    
    class ConsoleClient
    {
        static void Main(string[] args)
        {
            using (CloudDigitalAssetAccessConsoleClient.CloudDigitalStorageService.CloudDigitalAssetStorageWebServiceClient client  = new CloudDigitalAssetAccessConsoleClient.CloudDigitalStorageService.CloudDigitalAssetStorageWebServiceClient())
            {
                Console.WriteLine("Enter any text to start the execution. Enter 'quit' to exit ...");
                while (true)
                {
                    try
                    {
                        string message = Console.ReadLine();
                        if (message.Equals("quit", StringComparison.CurrentCultureIgnoreCase))
                            break;
                        
                        //// uploading a sample image
                        string uploadFolder = @"C:\MSDN\";
                        string filename = "DSC04085.jpg";
                        string filePath = Path.Combine(uploadFolder, filename);
                        string assetId = null; 
                        Console.WriteLine("Trying to upload File = " + filePath);
                        using (FileStream instream1 = File.OpenRead(filePath))
                        {
                            // STEP 1: store Asset in the cloud 
                            assetId = client.StoreAsset(instream1);
                        } 
                        Console.WriteLine("Upload Completed. Asset Id :" + assetId);

                        // STEP 2: Set additional meta-data with the Asset 
                        Console.WriteLine("Adding metadata on the asset..");
                        AssetMetadata metadata = new AssetMetadata();
                        metadata.AssetId = assetId;
                        metadata.ContentType = "image/pjpeg"; //"application/pdf";
                        Dictionary<string, string> collection = new Dictionary<string, string>();
                        collection.Add("location", "redmond, WA");
                        metadata.Metadata = collection;
                        client.SetAssetMetadata(metadata);
                        Console.WriteLine("Done: Added metadata on the asset: " + assetId);
                        
                        // STEP 3: Process the Asset 
                        Console.WriteLine("Requesting the worker to process the on the asset to generate thumbnails etc.");
                        client.ProcessAsset(assetId);
                        

                        // STEP 4: Retrieve Asset Metadata 
                        Console.WriteLine("Done: Worker process initiated for asset : " + assetId);
                        
                        Console.WriteLine("Retrieving metadata about asset : " + assetId);
                        AssetMetadata metaData = client.GetMetaData(assetId);
                        Console.WriteLine("Asset Name :" + metaData.AssetName);
                        Console.WriteLine("Asset URI :" + metaData.AssetUri);
                        Console.WriteLine("Asset ThumbNail:" + metaData.ThumbNailUri);

                        Dictionary<string, string> additionalMetadata = metaData.Metadata;
                        if (additionalMetadata != null)
                        {
                            Dictionary<string, string>.KeyCollection keyColl = metadata.Metadata.Keys;
                            foreach (string key in keyColl)
                            {
                                string val;
                                metadata.Metadata.TryGetValue(key, out val);
                                Console.WriteLine("Key:" + key+ "value :" + val);
                            }
                        }
                        // STEP 5: Retrieve All Asset Metadata 
                        Console.WriteLine("Retrieving metadata for all stored assets .. ");
                        AssetMetadata[] metaDataArry = client.GetAllAssetsMetaData();
                        if (metaDataArry != null)
                        {
                            var assetUri = from o in metaDataArry
                                       select ((AssetMetadata)o).AssetUri;

                            Console.WriteLine("Asset URIs:");
                            foreach (var tempAssetUri in assetUri)
                            {
                                Console.WriteLine(tempAssetUri.AbsoluteUri);
                            }
                        }
                        foreach (AssetMetadata data in metaDataArry)
                        {
                            Dictionary<string, string> dataVal = data.Metadata;
                            if (dataVal != null)
                            {
                                Console.WriteLine("Asset Name :" + data.AssetName);
                                Dictionary<string, string>.KeyCollection keyColl = data.Metadata.Keys;
                                foreach (string key in keyColl)
                                {
                                    string val;
                                    data.Metadata.TryGetValue(key, out val);
                                    
                                    Console.WriteLine("Key:" + key + "value :" + val);
                                }
                            }
                        }

                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.Message);
                    }
                }
            }
        }
    }
}
 