﻿// <copyright file="CloudDigitalAssetStorageService.svc.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>

namespace Samples.CloudDigitalAssetStorageWebService
{
    using Microsoft.ServiceHosting.ServiceRuntime;
    using System;
    using System.Collections; 
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Runtime.Serialization;
    using System.ServiceModel;
    using System.Text;
    using Samples.CloudDigitalAssetStorageService_Utilities;
    using Microsoft.Samples.ServiceHosting.StorageClient;
  
    // Asset Storage Web Service implementation 
    public class CloudDigitalAssetStorageWebService : ICloudDigitalAssetStorageWebService
    {
        // Test methods
        public void DoWork(string message)
        {
            RoleManager.WriteToLog("Information", string.Format("Message  '{0}'", message));
        }

        // Test methods
        public Stream GetAsset(string assetType)
        {
            return null;
        }

        // store any asset to the cloud storage and returns the asset identifier 
        public string StoreAsset(Stream asset)
        {
            RoleManager.WriteToLog("Information", string.Format("Message  '{0}'", "Begin Method - Store Asset"));
            MemoryStream outstream = null;
            string assetId = null;
            string assetTypeStr = "image/pjpeg";
                
            try
            {
                outstream = new MemoryStream();
                ////read from the input stream in 4K chunks and save to output stream
                const int bufferLen = 4096;
                byte[] buffer = new byte[bufferLen];
                int count = 0;
                while ((count = asset.Read(buffer, 0, bufferLen)) > 0)
                {
                    outstream.Write(buffer, 0, count);
                }
                outstream.Seek(0, SeekOrigin.Begin);

                
                ////Store data 
                assetId = StorageUtility.StoreImage(assetTypeStr, outstream);
                
                RoleManager.WriteToLog("Information", "Image is stored in the cloud");
            }
            catch (IOException ex)
            {
                RoleManager.WriteToLog("Information", string.Format("Exception is '{0}'", ex.ToString()));
            }
            finally
            {
                if (outstream != null)
                {
                    outstream.Close();
                    outstream.Dispose();
                }
                if (asset != null)
                {
                    asset.Close();
                    asset.Dispose();
                }
            }

            RoleManager.WriteToLog("Information", string.Format("Message  '{0}'", "End Method - Store Asset"));
            return assetId;
        }

        // associate meta data with a previously created asset 
        public bool SetAssetMetadata(AssetMetadata metadata)
        {
            if (metadata != null)
            {
                string assetId = metadata.AssetId;
              
                //// now process the asset 
                StorageUtility.UpdateAssetMetadata(assetId, metadata);
            }
            return false;
        }
        // process an asset ; for example create a thumbnail 
        public bool ProcessAsset(string assetId)
        {
            //// Process the asset 
            StorageUtility.ProcessAsset(assetId);
            return true;
        }
  
          
        // get all asset metadata 
        public IEnumerable<AssetMetadata> GetAllAssetsMetaData()
        {
            IEnumerable<AssetMetadata> metaData = StorageUtility.GetAllMetaData();
            return metaData;
        }

        // get asset specific metadata 
        public AssetMetadata GetMetaData(string assetId)
        {
            return StorageUtility.GetMetaData(assetId);
        }
    }
}
